/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.core;

import java.text.ParseException;
import java.util.Date;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;

public class Term implements Entity, HasGetID {
	private int id;
	private String name;
	private String comment;
	private Date start = null;
	private Date end = null;
	
	/**
	 * Constructor de los nuevos Term. Al añadir un Term desde la aplicación,
	 * se usará este constructor. Cuando una cadena sea nula el objeto pasará
	 * a representar la cadena vacía "".
	 * 
	 * @param idTerm
	 * @param name
	 * @param comment
	 * @param start
	 * @param end
	 */
	public Term (Integer idTerm, String name, String comment, Date start, Date end) {
		
		setID(idTerm);
		setName(name);
		setComment(comment);
		setStart(start);
		setEnd(end);		
	}
	
	/**
	 * Constructor de los Term importados. Al importar un Term desde un csv,
	 * se usará este constructor.
	 * @param name
	 * @param comment
	 * @param startString fecha en formato de cadena SQL
	 * @param endString
	 * @throws ParseException cuando una de las fechas no se puede interpretar (De acuerdo
	 * a la locale seleccionada en el fichero properties)
	 */
	public Term(String name, String comment, String startString, String endString) throws ParseException{
		this(0, name, comment, (startString.equals(""))?null:AttendancemanagerApplication.df.parse(startString), (endString.equals(""))?null:AttendancemanagerApplication.df.parse(endString));
		//this(0, name, comment, AttendancemanagerApplication.df.parse(startString), AttendancemanagerApplication.df.parse(endString));
	}
	
	/**
	 * Método estático que devuelve un array con las cabeceras del csv
	 * @return devuelve un array con las cadenas de las cabeceras que deberían tener los csv de Term
	 */
	public static String[] getBasicProperties() {
		String columns[] =  {"name", "comment", "start", "end"};
		return columns;
	}

	public int getID() {
		return id;
	}
	public void setID(int id) {
		this.id = id;
	}
	public String getComment() {
		return comment;
	}
	public void setComment(String comment) {
		this.comment = Constants.longString(comment);
	}
	public Date getStart() {
		return start;
	}
	public void setStart(java.util.Date start) {
		this.start = start;
	}
	public Date getEnd() {
		return end;
	}
	public void setEnd(java.util.Date end) {
		this.end = end;
	}
	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		this.name = Constants.shortString(name);
	}
	
	/**
	 * Se añade él mismo a la base de datos
	 * @return el id del Term insertado
	 */
	public int add() {
		int id = 0;
		
		try {
			id = storage.addTerm(this);
		} catch(Exception e) {
			Logger.log("Term.add() -> " + this);
			e.printStackTrace();
		}
		
		return id;
	}
	
	/**
	 * Se elimina él mismo de la base de datos
	 */
	public void delete() {
		 storage.deleteTerm(this.id);
	}
	
	/**
	 * Se modifica él mismo en la base de datos
	 * @return el código de retorno
	 */
	public int update() {
		return storage.updateTerm(this);
	}

	@Override
	public String toString() {
		return "Term [id=" + id + ", name=" + name + ", comment=" + comment
				+ ", start=" + start + ", end=" + end + "]";
	}
}
