/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.screenlayouts;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.ResourceBundle;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;
import org.iespuigcastellar.attendancemanager.core.CreatePDF;
import org.iespuigcastellar.attendancemanager.core.DatabaseStorage;
import org.iespuigcastellar.attendancemanager.core.Group;
import org.iespuigcastellar.attendancemanager.core.Logger;
import org.iespuigcastellar.attendancemanager.core.Mailer;
import org.iespuigcastellar.attendancemanager.core.SMS;
import org.iespuigcastellar.attendancemanager.core.NotificationReceipt;
import org.iespuigcastellar.attendancemanager.core.Student;
import org.iespuigcastellar.attendancemanager.core.UniqueIDSet;

import com.vaadin.data.Container;
import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.data.Container.ItemSetChangeEvent;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Panel;
import com.vaadin.ui.Table;
import com.vaadin.ui.TextField;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.AbstractSelect.Filtering;
import com.vaadin.ui.Button.ClickEvent;

/**
 * 
 * @author vcarceler
 *
 */
@SuppressWarnings("serial")
public class CommunicationLayout extends VerticalLayout {
	private AttendancemanagerApplication app = null;
	//private ResourceBundle rb = AttendancemanagerApplication.rb;
	private ResourceBundle locale = AttendancemanagerApplication.locale;
	private DatabaseStorage storage = AttendancemanagerApplication.storage;
	
	private boolean admin = false;
	
	private SMS sms = new SMS();
	private Mailer mail = new Mailer();
	
	private Table destinationTable;
	private ComboBox groupComboBox;
	private Button addgroupButton;
	private ComboBox studentComboBox;
	private Button addstudentButton;
	private Button removeButton;
	private Button removeallButton;
	private TextField smsTextField;
	private TextField mailsubjectTextField;
	private TextField mailbodyTextField;
	
	//private final int COLUMNS = 20;
	private final int ROWS = 10;
	
	public CommunicationLayout() {
		
	}
	
	public CommunicationLayout(boolean admin) {
		this.admin = admin;
	}
	
	public void attach() {
		app = (AttendancemanagerApplication)getApplication();
		
		initLayout();
		refreshLayout();
	}
	
	public void initLayout() {
		groupComboBox  = new ComboBox();
		groupComboBox.setInputPrompt(locale.getString("COMMUNICATIONLAYOUT_SELECTGROUP_CAPTION"));
		groupComboBox.setDescription(locale.getString("COMMUNICATIONLAYOUT_SELECTGROUP_CAPTION"));
		groupComboBox.setFilteringMode(Filtering.FILTERINGMODE_CONTAINS);
		groupComboBox.setImmediate(true);
		groupComboBox.addContainerProperty(DatabaseStorage.GROUPNAME_PROPERTY, String.class, "");
		groupComboBox.setItemCaptionPropertyId(DatabaseStorage.GROUPNAME_PROPERTY);
		groupComboBox.setNullSelectionAllowed(false);
		
		addgroupButton  = new Button(locale.getString("COMMUNICATIONLAYOUT_ADDGROUP_CAPTION"));
		addgroupButton.addListener(new Button.ClickListener() {
			
			@SuppressWarnings("unchecked")
			@Override
			public void buttonClick(ClickEvent event) {
				
				Group group = (Group)groupComboBox.getValue();
				List<Student> students = storage.getStudentsCollection(group.getID());
				
				// Obtenemos la colección de alumnos que hay que añadir a la tabla
				// evitando duplicados. Es decir, de los que quiero añadir aquellos
				// que no están ya en la tabla.
				
				// No es posible preguntar al DataSource de la tabla si contiene o no determinada
				// instancia, pues si la instancia a buscar es otra (aunque tenga el mismo valor)
				// que la contenida en el DataSource siempre retorna que no existe.
				// Es decir, compara instancias (mismo objeto) no valores.
				
				Collection<Student> tablestudents = (Collection<Student>)destinationTable.getContainerDataSource().getItemIds();
				
				UniqueIDSet<Student> uniqueidset = new UniqueIDSet<Student>();
				uniqueidset.addAll(tablestudents);
				
				Collection<Student> uniqs = uniqueidset.notInSet(students);
				
				for(Student student : uniqs) {
					
					Item item = destinationTable.addItem(student);
					item.getItemProperty(DatabaseStorage.SURNAME1_PROPERTY).setValue(student.getSurname1());
					item.getItemProperty(DatabaseStorage.SURNAME2_PROPERTY).setValue(student.getSurname2());
					item.getItemProperty(DatabaseStorage.NAME_PROPERTY).setValue(student.getName());
					item.getItemProperty(DatabaseStorage.EMAIL_PROPERTY).setValue(student.getEmail());
					item.getItemProperty(DatabaseStorage.SMSPHONE_PROPERTY).setValue(student.getSmsphone());
				}
			}
		});
		
		studentComboBox  = new ComboBox();
		studentComboBox.setInputPrompt(locale.getString("COMMUNICATIONLAYOUT_SELECTSTUDENT_CAPTION"));
		studentComboBox.setDescription(locale.getString("COMMUNICATIONLAYOUT_SELECTSTUDENT_CAPTION"));
		studentComboBox.setFilteringMode(Filtering.FILTERINGMODE_CONTAINS);
		studentComboBox.setImmediate(true);
		studentComboBox.addContainerProperty(DatabaseStorage.FULLNAME_PROPERTY, String.class, "");
		studentComboBox.setItemCaptionPropertyId(DatabaseStorage.FULLNAME_PROPERTY);
		studentComboBox.setNullSelectionAllowed(false);
		
		addstudentButton  = new Button(locale.getString("COMMUNICATIONLAYOUT_ADDSTUDENT_CAPTION"));
		addstudentButton.addListener(new Button.ClickListener() {
			
			@SuppressWarnings("unchecked")
			@Override
			public void buttonClick(ClickEvent event) {
				Student student = (Student)studentComboBox.getValue();
				
				UniqueIDSet<Student> uniqueidset = new UniqueIDSet<Student>();
				uniqueidset.addAll((Collection<Student>)destinationTable.getContainerDataSource().getItemIds());
				
				if(!uniqueidset.contains(student)) {
					Item item = destinationTable.addItem(student);
					item.getItemProperty(DatabaseStorage.SURNAME1_PROPERTY).setValue(student.getSurname1());
					item.getItemProperty(DatabaseStorage.SURNAME2_PROPERTY).setValue(student.getSurname2());
					item.getItemProperty(DatabaseStorage.NAME_PROPERTY).setValue(student.getName());
					item.getItemProperty(DatabaseStorage.EMAIL_PROPERTY).setValue(student.getEmail());
					item.getItemProperty(DatabaseStorage.SMSPHONE_PROPERTY).setValue(student.getSmsphone());
				}
			}
		});
		
		removeButton = new Button(locale.getString("COMMUNICATIONLAYOUT_QUITSTUDENT_CAPTION"));
		removeButton.setEnabled(false);
		removeButton.addListener(new Button.ClickListener() {
			
			@SuppressWarnings("unchecked")
			@Override
			public void buttonClick(ClickEvent event) {
				for(Student student : (Collection<Student>)destinationTable.getValue()) {
					destinationTable.removeItem(student);
				}
			}
		});
		
		removeallButton = new Button(locale.getString("COMMUNICATIONLAYOUT_CLEAR_CAPTION"));
		removeallButton.setEnabled(false);
		removeallButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				destinationTable.removeAllItems();
			}
		});
		
		destinationTable = new Table();
		destinationTable.setImmediate(true);
		destinationTable.setSelectable(true);
		destinationTable.setMultiSelect(true);
		destinationTable.setNullSelectionAllowed(true);
		destinationTable.setColumnReorderingAllowed(true);
		destinationTable.setColumnCollapsingAllowed(true);
		destinationTable.setStyleName("striped");
		destinationTable.setSizeFull();
		
		destinationTable.addContainerProperty(DatabaseStorage.SURNAME1_PROPERTY, String.class, "", locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SURNAME1"), null,Table.ALIGN_LEFT );
		destinationTable.addContainerProperty(DatabaseStorage.SURNAME2_PROPERTY, String.class, "", locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SURNAME2"), null, Table.ALIGN_LEFT);
		destinationTable.addContainerProperty(DatabaseStorage.NAME_PROPERTY, String.class, "", locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"), null, Table.ALIGN_LEFT);
		destinationTable.addContainerProperty(DatabaseStorage.EMAIL_PROPERTY, String.class, "",locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_EMAIL"), null, Table.ALIGN_LEFT);
		destinationTable.addContainerProperty(DatabaseStorage.SMSPHONE_PROPERTY, String.class, "",locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SMSPHONE"), null,Table.ALIGN_LEFT);
		
		destinationTable.addListener(new Property.ValueChangeListener() {
			
			@SuppressWarnings("rawtypes")
			@Override
			public void valueChange(ValueChangeEvent event) {
				Collection value = (Collection)destinationTable.getValue();
				if(value.size()>0) {
					removeButton.setEnabled(true);
				} else {
					removeButton.setEnabled(false);
				}
			}
		});
		
		destinationTable.addListener(new Container.ItemSetChangeListener() {
			
			@Override
			public void containerItemSetChange(ItemSetChangeEvent event) {
				if(destinationTable.getContainerDataSource().size()>0) {
					removeallButton.setEnabled(true);
				} else {
					removeallButton.setEnabled(false);
				}
			}
		});
		
		VerticalLayout v1Layout = new VerticalLayout();
		v1Layout.setSizeUndefined();
		v1Layout.setSpacing(true);
		
		Panel groupPanel = new Panel(locale.getString("COMMUNICATIONLAYOUT_GROUPPANEL_CAPTION"));
		groupPanel.setStyleName("bubble");
		groupPanel.addComponent(groupComboBox);
		groupPanel.addComponent(addgroupButton);
		groupPanel.setSizeUndefined(); // Prueba FF
		groupPanel.getContent().setSizeUndefined(); //
		
		Panel studentPanel = new Panel(locale.getString("COMMUNICATIONLAYOUT_STUDENTPANEL_CAPTION"));
		studentPanel.setStyleName("bubble");
		studentPanel.addComponent(studentComboBox);
		studentPanel.addComponent(addstudentButton);
		studentPanel.setSizeUndefined(); // Prueba FF
		studentPanel.getContent().setSizeUndefined(); //
		
		v1Layout.addComponent(groupPanel);
		v1Layout.addComponent(studentPanel);
		v1Layout.addComponent(removeButton);
		v1Layout.addComponent(removeallButton);
		
		Panel destinationPanel = new Panel(locale.getString("COMMUNICATIONLAYOUT_DESTINATIONS_CAPTION"));
		destinationPanel.setStyleName("bubble");
		destinationPanel.setSizeFull();
		HorizontalLayout h1Layout = new HorizontalLayout();
		h1Layout.setSizeFull();
		h1Layout.setSpacing(true);
		h1Layout.addComponent(destinationTable);
		h1Layout.setExpandRatio(destinationTable, 1.0f);
		h1Layout.addComponent(v1Layout);
		//destinationPanel.setContent(h1Layout);
		destinationPanel.addComponent(h1Layout);
		destinationPanel.getContent().setSizeFull();
		
		Panel smsPanel = new Panel(locale.getString("COMMUNICATIONLAYOUT_SMSPANEL_CAPTION"));
		smsPanel.setStyleName("bubble");
		smsPanel.setSizeFull();
		smsPanel.getContent().setSizeFull(); // Corregir problema en layout
		
		smsTextField = new TextField(locale.getString("COMMUNICATIONLAYOUT_SMSTEXTFIELD_CAPTION") + sms.MAX_LENGTH);
		smsTextField.setMaxLength(sms.MAX_LENGTH);
		smsTextField.setSizeFull();
		//smsTextField.setRows(ROWS);
		
		final Button sendsmsButton = new Button(locale.getString("COMMUNICATIONLAYOUT_SENDSMS_CAPTION"));
		sendsmsButton.addListener(new Button.ClickListener() {
			
			@SuppressWarnings("unchecked")
			@Override
			public void buttonClick(ClickEvent event) {
				
				Logger.log("CommunicationLayout: Send SMSs");
				
				List<NotificationReceipt> receipts = new ArrayList<NotificationReceipt>();
				
				for(Student student : (Collection<Student>)destinationTable.getContainerDataSource().getItemIds()) {
					boolean status = sms.sendSMS(student.getSmsphone(), (String)smsTextField.getValue());
					receipts.add(new NotificationReceipt(student.getSmsphone(), status, (String)smsTextField.getValue()));
				}
				
				CreatePDF pdf = new CreatePDF();
				app.DownloadFile(sendsmsButton.getWindow(), pdf.smsList(receipts), "sms-" + System.currentTimeMillis() + ".pdf", AttendancemanagerApplication.MIME_PDF);
			}
		});
		
		Button clearsmsButton = new Button(locale.getString("COMMUNICATIONLAYOUT_CLEAR_CAPTION"));
		clearsmsButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				smsTextField.setValue("");
			}
		});
		
		HorizontalLayout smsbuttonsLayout = new HorizontalLayout();
		smsbuttonsLayout.addComponent(sendsmsButton);
		smsbuttonsLayout.addComponent(clearsmsButton);
		
		VerticalLayout smsVerticalLayout = new VerticalLayout();
		smsVerticalLayout.setSizeFull();
		smsVerticalLayout.addComponent(smsTextField);
		smsVerticalLayout.setExpandRatio(smsTextField, 1.0f);
		smsVerticalLayout.addComponent(smsbuttonsLayout);
//		smsPanel.addComponent(smsTextField);
//		smsPanel.addComponent(smsbuttonsLayout);
		smsPanel.addComponent(smsVerticalLayout);
		
		Panel mailPanel = new Panel(locale.getString("COMMUNICATIONLAYOUT_MAILPANEL_CAPTION"));
		mailPanel.setStyleName("bubble");
		mailPanel.setSizeFull();
		mailPanel.getContent().setSizeFull(); // Corregir problema en layout
		
		mailsubjectTextField = new TextField(locale.getString("COMMUNICATIONLAYOUT_MAILSUBJECT_CAPTION"));
		mailsubjectTextField.setMaxLength(255);
		mailsubjectTextField.setWidth("100%");
		mailbodyTextField = new TextField(locale.getString("COMMUNICATIONLAYOUT_MAILBODY_CAPTION"));
		mailbodyTextField.setMaxLength(4096);
		mailbodyTextField.setSizeFull();
		//mailbodyTextField.setRows(ROWS);
		
		final Button sendmailButton = new Button(locale.getString("COMMUNICATIONLAYOUT_SENDMAIL_CAPTION"));
		sendmailButton.addListener(new Button.ClickListener() {
			
			@SuppressWarnings("unchecked")
			@Override
			public void buttonClick(ClickEvent event) {
				
				Logger.log("CommunicationLayout: Send mails");
				
				mail.setSubject((String)mailsubjectTextField.getValue());
				mail.setBodyText((String)mailbodyTextField.getValue());
				
				List<NotificationReceipt> receipts = new ArrayList<NotificationReceipt>();
				
				for(Student student : (Collection<Student>)destinationTable.getContainerDataSource().getItemIds()) {
					mail.setTo(student.getEmail());
					boolean status = mail.sendMail();
					
					receipts.add(new NotificationReceipt(student.getEmail(), status, (String)mailbodyTextField.getValue()));
				}
				
				CreatePDF pdf = new CreatePDF();
				app.DownloadFile(getWindow(), pdf.mailList(receipts), "mail-" + System.currentTimeMillis() + ".pdf", AttendancemanagerApplication.MIME_PDF);
			}
		});
		
		Button clearmailButton = new Button(locale.getString("COMMUNICATIONLAYOUT_CLEAR_CAPTION"));
		clearmailButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				mailsubjectTextField.setValue("");
				mailbodyTextField.setValue("");
			}
		});
		
		HorizontalLayout mailbuttonsLayout = new HorizontalLayout();
		mailbuttonsLayout.addComponent(sendmailButton);
		mailbuttonsLayout.addComponent(clearmailButton);
		
		VerticalLayout mailVerticalLayout = new VerticalLayout();
		mailVerticalLayout.setSizeFull();
		mailVerticalLayout.addComponent(mailsubjectTextField);
		mailVerticalLayout.addComponent(mailbodyTextField);
		mailVerticalLayout.setExpandRatio(mailbodyTextField, 1.0f);
		mailVerticalLayout.addComponent(mailbuttonsLayout);
//		mailPanel.addComponent(mailsubjectTextField);
//		mailPanel.addComponent(mailbodyTextField);
//		mailPanel.addComponent(mailbuttonsLayout);
		mailPanel.addComponent(mailVerticalLayout);
		
		
		
		HorizontalLayout h2Layout = new HorizontalLayout();
		h2Layout.setSizeFull();
		h2Layout.setSpacing(true);
		h2Layout.addComponent(smsPanel);
		h2Layout.addComponent(mailPanel);
		
		setSizeFull();
		setSpacing(true);
		
		addComponent(destinationPanel);
		addComponent(h2Layout);
		
		setExpandRatio(destinationPanel, 0.6f);
		setExpandRatio(h2Layout, 0.4f);
	}
	
	public void refreshLayout() {
		loadCombos();
	}
	
	public void loadCombos() {
		groupComboBox.removeAllItems();
		List<Group> groups = null;
		
		if(admin) {
			groups = storage.allGroups();
		} else {
			groups = storage.allGroups(app.user);
		}
		
		for(Group group : groups) {
			Item item = groupComboBox.addItem(group);
			if(item != null) item.getItemProperty(DatabaseStorage.GROUPNAME_PROPERTY).setValue(group.getName());
		}
		
		if(!groups.isEmpty()) {
			groupComboBox.select(groups.get(0));
		}
		
		studentComboBox.removeAllItems();
		List<Student> students = null;
		
		if(admin) {
			students = storage.getStudentsCollection(0);
		} else {
			students = storage.getStudentsCollection(0, app.user.getID());
		}
		
		for(Student student : students) {
			Item item = studentComboBox.addItem(student);
			if(item != null) item.getItemProperty(DatabaseStorage.FULLNAME_PROPERTY).setValue(student.getFullName());
		}
		
		if(!students.isEmpty()) {
			studentComboBox.select(students.get(0));
		}
	}
}
