/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

/*
 * A layout with teacher's tabs
 */

package org.iespuigcastellar.attendancemanager.screenlayouts;

import java.util.ResourceBundle;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;

import com.vaadin.terminal.ThemeResource;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.TabSheet;

@SuppressWarnings("serial")
public class TeacherTabsLayout extends VerticalLayout {
	
	private AttendancemanagerApplication app = null;
	
	private TabSheet tabsTabSheet = new TabSheet();
	
	private TeacherMainLayout teachermainlayout = new TeacherMainLayout();
	private ReportLayout reportlayout = new ReportLayout();
	
	private ResourceBundle locale = AttendancemanagerApplication.locale;
	
	private void initLayout() {
		
		tabsTabSheet.setSizeFull();
		
		TabSheet.Tab tab;

		tab = tabsTabSheet.addTab(teachermainlayout, locale.getString("ADMINLAYOUT_TEACHERMAINTAB_CAPTION"), new ThemeResource("img/house.png"));
		tab.setDescription(locale.getString("ADMINLAYOUT_TEACHERMAINTAB_DESCRIPTION"));
 		
 		tab = tabsTabSheet.addTab(reportlayout, locale.getString("ADMINLAYOUT_REPORTSTAB_CAPTION"), new ThemeResource("img/report.png"));
		tab.setDescription(locale.getString("ADMINLAYOUT_REPORTSTAB_DESCRIPTION"));
 		
 		if(AttendancemanagerApplication.storage.isTutor(app.user)) {
 			JustifyMissesLayout justifymisseslayout = new JustifyMissesLayout();
 			tab = tabsTabSheet.addTab(justifymisseslayout, locale.getString("ADMINLAYOUT_JUSTIFYMANAGERTAB_CAPTION"), null);
 			tab.setDescription(locale.getString("ADMINLAYOUT_JUSTIFYMANAGERTAB_DESCRIPTION"));
 	 		tab.setIcon(new ThemeResource("img/flag_yellow.png"));
 			
 	 		CommunicationLayout communicationLayout = new CommunicationLayout();
 	 		tab = tabsTabSheet.addTab(communicationLayout, locale.getString("ADMINLAYOUT_MESSAGESTAB_CAPTION"), null);
 	 		tab.setDescription(locale.getString("ADMINLAYOUT_MESSAGESMANAGERTAB_DESCRIPTION"));
 	 		tab.setIcon(new ThemeResource("img/email.png"));
 	 		
// 			MessageLayout messagelayout = new MessageLayout();
// 			tab = tabsTabSheet.addTab(messagelayout, locale.getString("ADMINLAYOUT_MESSAGESTAB_CAPTION"), null);
// 			tab.setDescription(locale.getString("ADMINLAYOUT_MESSAGESMANAGERTAB_DESCRIPTION"));
// 	 		tab.setIcon(new ThemeResource("img/email.png"));
 	 		
 	 		NotifierEmailSmsLayout notifieremailsmslayout = new NotifierEmailSmsLayout(false);
 			tab = tabsTabSheet.addTab(notifieremailsmslayout, locale.getString("ADMINLAYOUT_NOTIFIERSMSEMAILTAB_CAPTION"), null);
 			tab.setDescription(locale.getString("ADMINLAYOUT_NOTIFIERSMSEMAILTAB_DESCRIPTION"));
 	 		tab.setIcon(new ThemeResource("img/bell.png"));
 		}
 		
		setSizeFull();
		tabsTabSheet.setSizeFull();
	}
	
	public void attach() {
		app = (AttendancemanagerApplication)getApplication();
		initLayout();
		addComponent(tabsTabSheet);
		this.setExpandRatio(tabsTabSheet, 1.0f);
		setSpacing(true);
	}

}
