/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.core;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * Provides a minimal implementation compatible with Set<HasGetID> in
 * which only unique IDs are allowed.
 * 
 * @author vcarceler
 */

public class UniqueIDSet<T extends HasGetID> {

	private LinkedHashMap<Integer, T> map = new LinkedHashMap<Integer, T>();

	public boolean add(T o) {
		boolean result = map.containsKey(o.getID());
		if(!result) map.put(o.getID(), o);
	
		return result;
	}

	public boolean addAll(Collection<T> c) {
		boolean result = false;
		
		for(T o : c) {
			if(!result && map.containsKey(o.getID())) result = true;
			if(!map.containsKey(o.getID())) map.put(o.getID(), o);
		}
		
		return result;
	}

	
	public void clear() {
		map.clear();		
	}

	public boolean contains(T o) {
		return map.containsKey(o.getID());
	}
	
	public Collection<T> getCollection() {
		return map.values();
	}
	
	/**
	 * Returns a collection with elements in c not in set.
	 * @param c
	 * @return
	 */
	public Collection<T> notInSet(Collection<T> c) {
		List<T> result = new ArrayList<T>();
		
		for(T e : c) {
			if(!map.containsKey(e.getID())) result.add(e);
		}
		
		return result;
	}
//
//	@Override
//	public boolean containsAll(Collection<?> arg0) {
//		// TODO Auto-generated method stub
//		return false;
//	}
//
//	@Override
//	public boolean isEmpty() {
//		// TODO Auto-generated method stub
//		return false;
//	}
//
//	@Override
//	public Iterator<HasGetID> iterator() {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public boolean remove(Object arg0) {
//		// TODO Auto-generated method stub
//		return false;
//	}
//
//	@Override
//	public boolean removeAll(Collection<?> arg0) {
//		// TODO Auto-generated method stub
//		return false;
//	}
//
//	@Override
//	public boolean retainAll(Collection<?> arg0) {
//		// TODO Auto-generated method stub
//		return false;
//	}
//
//	@Override
//	public int size() {
//		// TODO Auto-generated method stub
//		return 0;
//	}
//
//	@Override
//	public Object[] toArray() {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public <T> T[] toArray(T[] arg0) {
//		// TODO Auto-generated method stub
//		return null;
//	}
	
//	public UniqueIDSet() {
//		
//	}
//	
//	public UniqueIDSet(Collection<HasGetID> c) {
//		for(HasGetID o : c) {
//			map.put(o.getID(), o);
//		}
//	}
//	

	
}
