/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

/*
 *	GroupManagerLayout designed by Xavier Junyent <xavierjunyent@yahoo.es>
 *
 *	GroupManagerLayout allows group administration.
 */

package org.iespuigcastellar.attendancemanager.screenlayouts;

import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.ResourceBundle;
import java.util.Set;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;
import org.iespuigcastellar.attendancemanager.core.CSVUploader;
import org.iespuigcastellar.attendancemanager.core.Constants;
import org.iespuigcastellar.attendancemanager.core.DatabaseStorage;
import org.iespuigcastellar.attendancemanager.core.Group;

import org.iespuigcastellar.attendancemanager.core.Teacher;

import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.util.BeanItem;
import com.vaadin.data.util.IndexedContainer;
import com.vaadin.data.validator.StringLengthValidator;
import com.vaadin.terminal.ThemeResource;
import com.vaadin.ui.AbstractSelect;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.Component;
import com.vaadin.ui.CssLayout;
import com.vaadin.ui.DefaultFieldFactory;
import com.vaadin.ui.Embedded;
import com.vaadin.ui.Field;
import com.vaadin.ui.Form;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.Panel;
import com.vaadin.ui.SplitPanel;
import com.vaadin.ui.Table;
import com.vaadin.ui.TextField;
import com.vaadin.ui.Upload;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Upload.FailedEvent;
import com.vaadin.ui.Upload.SucceededEvent;

@SuppressWarnings("serial")
public class GroupManagerLayout extends VerticalLayout {
	
	private AttendancemanagerApplication app;
	private ResourceBundle locale = AttendancemanagerApplication.locale;
	private DatabaseStorage storage = AttendancemanagerApplication.storage;
	
	private Button newButton;
	private Button editButton;
	private Button deleteButton;
	private Button includeButton;
	private Button importButton;
	private Button addstudentButton;
	private Button removestudentButton;
	
	private Table groupsTable;
	private Table studentsGroupTable;
	private Table studentsNoGroupTable;
	
	private NewGroupWindow newgroupWindow;
	private EditGroupWindow editgroupWindow;
	private DeleteGroupWindow deletegroupWindow;
	private IncludeGroupWindow includegroupWindow;
	private ImportGroupWindow importgroupWindow;
	
	
	private void initLayout() {

		addstudentButton= new Button();
		addstudentButton.setIcon(new ThemeResource("img/24-em-up.png"));
		addstudentButton.setEnabled(false);
		addstudentButton.setWidth("100%");
		addstudentButton.setStyleName("small");
		
		removestudentButton= new Button();
		removestudentButton.setIcon(new ThemeResource("img/24-em-down.png"));
		removestudentButton.setEnabled(false);
		removestudentButton.setWidth("100%");
		removestudentButton.setStyleName("small");
		
		newButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_NEWBUTTON_CAPTION"));
		newButton.setIcon(new ThemeResource("img/group_add.png"));
		newButton.setStyleName("default");
		
	
		editButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_EDITBUTTON_CAPTION"));
		editButton.setIcon(new ThemeResource("img/group_edit.png"));
		editButton.setEnabled(false);
		editButton.setStyleName("default");
		
		deleteButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_DELETEBUTTON_CAPTION"));
		deleteButton.setIcon(new ThemeResource("img/group_delete.png"));
		deleteButton.setEnabled(false);
		deleteButton.setStyleName("default");
		
		includeButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_INCLUDEBUTTON_CAPTION"));
		includeButton.setDescription(app.locale.getString("GROUPMANAGERLAYOUT_INCLUDEBUTTON_DESCRIPTION"));
		includeButton.setIcon(new ThemeResource("img/user_add.png"));
		includeButton.setEnabled(false);
		includeButton.setStyleName("default");
		
		importButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_IMPORTBUTTON_CAPTION"));
		importButton.setIcon(new ThemeResource("img/group_gear.png"));
		importButton.setStyleName("default");
		
		CssLayout toolbargroups = new CssLayout();
		toolbargroups.addComponent(newButton);
		toolbargroups.addComponent(editButton);
		toolbargroups.addComponent(deleteButton);
		toolbargroups.addComponent(includeButton);
		toolbargroups.addComponent(importButton);
		toolbargroups.setStyleName("toolbar");
		toolbargroups.setWidth("100%");

		HorizontalLayout toolbarstudentsnogroup = new HorizontalLayout();
		toolbarstudentsnogroup.addComponent(addstudentButton);
		toolbarstudentsnogroup.setSpacing(true);
		toolbarstudentsnogroup.setMargin(false);
		toolbarstudentsnogroup.setWidth("100%");
		toolbarstudentsnogroup.setExpandRatio(addstudentButton, 1);
		toolbarstudentsnogroup.setComponentAlignment(addstudentButton, Alignment.MIDDLE_CENTER);
		
		HorizontalLayout toolbarstudentsgroup = new HorizontalLayout();
		toolbarstudentsgroup.addComponent(removestudentButton);
		toolbarstudentsgroup.setSpacing(true);
		toolbarstudentsgroup.setMargin(false);
		toolbarstudentsgroup.setWidth("100%");
		toolbarstudentsgroup.setExpandRatio(removestudentButton, 1);
		toolbarstudentsgroup.setComponentAlignment(removestudentButton, Alignment.MIDDLE_CENTER);
		
		groupsTable = new Table();
		groupsTable.setImmediate(true);
		groupsTable.setColumnReorderingAllowed(true);
		groupsTable.setColumnCollapsingAllowed(true);
		groupsTable.setStyleName("striped");
		groupsTable.setSizeFull();
		
		studentsGroupTable = new Table();
		studentsGroupTable.setImmediate(true);
		studentsGroupTable.setColumnReorderingAllowed(true);
		studentsGroupTable.setColumnCollapsingAllowed(true);
		studentsGroupTable.setStyleName("striped");
		studentsGroupTable.setSizeFull();
		
		studentsNoGroupTable = new Table();
		studentsNoGroupTable.setImmediate(true);
		studentsNoGroupTable.setColumnReorderingAllowed(true);
		studentsNoGroupTable.setColumnCollapsingAllowed(true);
		studentsNoGroupTable.setStyleName("striped");
		studentsNoGroupTable.setSizeFull();

		
		Panel groupsPanel = new Panel(app.locale.getString("GROUPMANAGERLAYOUT_GROUPSPANEL_CAPTION"));
		groupsPanel.addComponent(groupsTable);
		groupsPanel.setSizeFull();
		groupsPanel.setStyleName("bubble");
		groupsPanel.setIcon(new ThemeResource("img/group.png"));
		groupsPanel.getContent().setSizeFull();
		
		Panel studentsGroupPanel = new Panel(app.locale.getString("GROUPMANAGERLAYOUT_STUDENTSGROUPPANEL_CAPTION"));
		Panel studentsNoGroupPanel = new Panel(app.locale.getString("GROUPMANAGERLAYOUT_STUDENTSNOGROUPPANEL_CAPTION"));
		
		studentsGroupPanel.addComponent(studentsGroupTable);
		studentsGroupPanel.addComponent(toolbarstudentsgroup);
		studentsGroupPanel.setSizeFull();
		studentsGroupPanel.setStyleName("bubble");
		studentsGroupPanel.setIcon(new ThemeResource("img/user.png"));
		studentsGroupPanel.getContent().setSizeFull();
		
		studentsNoGroupPanel.addComponent(studentsNoGroupTable);
		studentsNoGroupPanel.setSizeFull();
		studentsNoGroupPanel.setStyleName("bubble");
		studentsNoGroupPanel.setIcon(new ThemeResource("img/user.png"));
		studentsNoGroupPanel.getContent().setSizeFull();
		
		VerticalLayout vl =  (VerticalLayout) studentsNoGroupPanel.getContent();
		vl.setMargin(false);
		vl.setExpandRatio(studentsNoGroupTable, 1.0f);
		vl.setSpacing(false);

		
		vl=(VerticalLayout) studentsGroupPanel.getContent();
		vl.setMargin(false);
		vl.setExpandRatio(studentsGroupTable, 1.0f);
		vl.setSpacing(false);
	
		
		vl =  (VerticalLayout) groupsPanel.getContent();
		vl.setMargin(false);
		vl.setExpandRatio(groupsTable, 1.0f);
		vl.setSpacing(false);
		
		//We want toolbarstudentsnogroup must be on top
		
		VerticalLayout studentsnogroupLayout = new VerticalLayout();
		studentsnogroupLayout.addComponent(toolbarstudentsnogroup);
		studentsnogroupLayout.addComponent(studentsNoGroupPanel);
		studentsnogroupLayout.setSizeFull();
		studentsnogroupLayout.setExpandRatio(studentsNoGroupPanel, 1);
		
		SplitPanel tablesStudentsSplitPanel = new SplitPanel(SplitPanel.ORIENTATION_VERTICAL);
		tablesStudentsSplitPanel.addComponent(studentsGroupPanel);
		tablesStudentsSplitPanel.addComponent(studentsnogroupLayout);
		tablesStudentsSplitPanel.setSizeFull();
		
		
		
		SplitPanel tablesSplitPanel = new SplitPanel(SplitPanel.ORIENTATION_HORIZONTAL);
		tablesSplitPanel.addComponent(groupsPanel);
		tablesSplitPanel.addComponent(tablesStudentsSplitPanel);
		tablesSplitPanel.setSizeFull();
		
		addComponent(toolbargroups);
		addComponent(tablesSplitPanel);
		
		setSizeFull();
		setMargin(false, true, false, true);
		
		setExpandRatio(tablesSplitPanel, 1.0f);
		setSpacing(true);
	}
	public void attach() {
		app = (AttendancemanagerApplication)getApplication();
		initLayout();
		groupsTable.addContainerProperty(DatabaseStorage.NAME_PROPERTY, String.class, "");
		groupsTable.addContainerProperty(DatabaseStorage.COMMENT_PROPERTY, String.class, "");
		groupsTable.addContainerProperty(DatabaseStorage.TEACHER_IDTEACHER_PROPERTY, String.class, "");
		groupsTable.addContainerProperty(DatabaseStorage.MANAGERTEACHER_PROPERTY, String.class, "");
		
		groupsTable.addContainerProperty(DatabaseStorage.STUDENTS_PROPERTY, Integer.class, 0);
		
		String visiblecolumns[]={
				DatabaseStorage.NAME_PROPERTY,
				DatabaseStorage.COMMENT_PROPERTY,
				DatabaseStorage.MANAGERTEACHER_PROPERTY
				};
		groupsTable.setVisibleColumns(visiblecolumns);
		groupsTable.setColumnHeaders(new String[] { 
				app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"),
				app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_COMMENT"),
				app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_MANAGERTEACHER") 
		});
		
//		app.storage.refreshTable(app.storage.getGroupsQueryContainer(), groupsTable);
		groupsTable.setSelectable(true);
		groupsTable.setMultiSelect(false);
		groupsTable.setImmediate(true);
		studentsGroupTable.addContainerProperty(DatabaseStorage.SURNAME1_PROPERTY, String.class, "");
		studentsGroupTable.addContainerProperty(DatabaseStorage.SURNAME2_PROPERTY, String.class, "");
		studentsGroupTable.addContainerProperty(DatabaseStorage.NAME_PROPERTY, String.class, "");
		studentsGroupTable.addContainerProperty(DatabaseStorage.EMAIL_PROPERTY, String.class, "");
		studentsGroupTable.addContainerProperty(DatabaseStorage.SMSPHONE_PROPERTY, String.class, "");
		studentsGroupTable.setColumnHeaders(new String[] { 
				app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SURNAME1"),
				app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SURNAME2"),
				app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"),
				app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_EMAIL"),
				app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SMSPHONE") 
		});
		//app.storage.refreshTable(app.storage.getStudentsQueryContainer(), studentsGroupTable);
		studentsGroupTable.setSelectable(true);
		studentsGroupTable.setMultiSelect(true);
		studentsGroupTable.setImmediate(true);
		studentsGroupTable.addListener(new Property.ValueChangeListener() {
			
			@Override
			public void valueChange(ValueChangeEvent event) {

				Set<?> value = (Set<?>) event.getProperty().getValue();
				if (null == value || value.size() == 0) {
					removestudentButton.setEnabled(false);

				} else {
					removestudentButton.setEnabled(true);
				}
				
			}
		});
		
		studentsNoGroupTable.addContainerProperty(DatabaseStorage.SURNAME1_PROPERTY, String.class, "");
		studentsNoGroupTable.addContainerProperty(DatabaseStorage.SURNAME2_PROPERTY, String.class, "");
		studentsNoGroupTable.addContainerProperty(DatabaseStorage.NAME_PROPERTY, String.class, "");
		studentsNoGroupTable.addContainerProperty(DatabaseStorage.EMAIL_PROPERTY, String.class, "");
		studentsNoGroupTable.addContainerProperty(DatabaseStorage.SMSPHONE_PROPERTY, String.class, "");
		studentsNoGroupTable.setColumnHeaders(new String[] { 
				app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SURNAME1"),
				app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SURNAME2"),
				app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"),
				app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_EMAIL"),
				app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SMSPHONE") 
		});
		//app.storage.refreshTable(app.storage.getStudentsQueryContainer(), studentsNoGroupTable);
		studentsNoGroupTable.setSelectable(true);
		studentsNoGroupTable.setMultiSelect(true);
		studentsNoGroupTable.setImmediate(true);
		studentsNoGroupTable.addListener(new Property.ValueChangeListener() {
			
			@Override
			public void valueChange(ValueChangeEvent event) {
				Set<?> value = (Set<?>) event.getProperty().getValue();
				if (null == value || value.size() == 0) {
					addstudentButton.setEnabled(false);

				} else {
					addstudentButton.setEnabled(true);
				}
				
			}
		});
		groupsTable.addListener(new Property.ValueChangeListener() {
			
			@Override
			public void valueChange(ValueChangeEvent event) {
				Object value = groupsTable.getValue();
				if (value!=null){
					refreshStudents((Integer)value);
					editButton.setEnabled(true);
					deleteButton.setEnabled(true);
					includeButton.setEnabled(true);
				} else {
					studentsGroupTable.removeAllItems();
					studentsNoGroupTable.removeAllItems();
					editButton.setEnabled(false);
					deleteButton.setEnabled(false);
					includeButton.setEnabled(false);
					removestudentButton.setEnabled(false);
					addstudentButton.setEnabled(false);
					
				}
			}
		});

		newgroupWindow = new NewGroupWindow();
		newgroupWindow.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CAPTION"));
		
		editgroupWindow = new EditGroupWindow();
		editgroupWindow.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_EDITGROUPWINDOW_CAPTION"));
		
		deletegroupWindow= new DeleteGroupWindow();
		deletegroupWindow.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_DELETEGROUPWINDOW_CAPTION"));
		
		try {
			importgroupWindow= new ImportGroupWindow();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		}
		importgroupWindow.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_IMPORTGROUPWINDOW_CAPTION"));
		
		newButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// Open window if not open already
				if(newgroupWindow.getParent() != null) {
					// Window already open
				} else {
					// open window
					//Refresh teachers list combo 
					refreshTeachers();
					newgroupWindow.setValuesEmpty();
					getWindow().addWindow(newgroupWindow);
				}
			}
		});
		
		editButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {

				Integer idItem=(Integer) groupsTable.getValue();
				Item item=groupsTable.getItem(idItem);
				Integer idGroup=(Integer) groupsTable.getValue();
				String name=(String) item.getItemProperty(DatabaseStorage.NAME_PROPERTY).getValue();
				String comment=(String) item.getItemProperty(DatabaseStorage.COMMENT_PROPERTY).getValue();
				String idteacherString = (String)item.getItemProperty(DatabaseStorage.TEACHER_IDTEACHER_PROPERTY).getValue();
				Integer idteacher;
				if ("".equals(idteacherString)) {idteacher=0;}else{idteacher=Integer.parseInt(idteacherString);}
				refreshTeachers();
			    editgroupWindow.setIdGroup(idGroup);
			    editgroupWindow.groupForm.getField("name").setValue(name);
		        editgroupWindow.groupForm.getField("comment").setValue(comment);
				editgroupWindow.groupForm.getField("teacher").setValue(idteacher);
				editgroupWindow.groupForm.getField("name").setCaption(app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"));
				editgroupWindow.groupForm.getField("comment").setCaption(app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_COMMENT"));
				editgroupWindow.groupForm.getField("teacher").setCaption(app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_MANAGERTEACHER"));
				getWindow().addWindow(editgroupWindow);
			}
		});
		
		deleteButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {

				Integer idItem=(Integer) groupsTable.getValue();
				Item item=groupsTable.getItem(idItem);
				Integer idGroup=(Integer) groupsTable.getValue();
				String name=(String) item.getItemProperty(DatabaseStorage.NAME_PROPERTY).getValue();
				String comment=(String) item.getItemProperty(DatabaseStorage.COMMENT_PROPERTY).getValue();
				String idteacherString = (String)item.getItemProperty(DatabaseStorage.TEACHER_IDTEACHER_PROPERTY).getValue();
				Integer idteacher;
				if ("".equals(idteacherString)) {idteacher=0;}else{idteacher=Integer.parseInt(idteacherString);}
				deletegroupWindow.setGroup(new Group(idGroup,name,comment,idteacher));
				getWindow().addWindow(deletegroupWindow);
			}
		});
		
		includeButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				includegroupWindow = new IncludeGroupWindow();
				includegroupWindow.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_INCLUDEGROUPWINDOW_CAPTION"));
				
				getWindow().addWindow(includegroupWindow);				
			}
		});
		
		importButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {

				getWindow().addWindow(importgroupWindow);
			}
		});
		
		addstudentButton.addListener(new Button.ClickListener() {
			@Override
			public void buttonClick(ClickEvent event) {

				Set<?> value = (Set<?>) studentsNoGroupTable.getValue();
				System.out.println(value);
				if (null != value && value.size() > 0) {
					Iterator it=value.iterator();
					Integer idgroup =(Integer) groupsTable.getValue();
					while(it.hasNext()){
						
						Integer idstudent=(Integer) it.next();
						System.out.println(idgroup + " " + idstudent);
						app.storage.addStudentToGroup(idstudent,idgroup);
					}
					Integer itemid=(Integer) groupsTable.getValue();
					Integer valuestudents =(Integer) groupsTable.getContainerProperty(itemid,"students").getValue();
					Integer newvaluestudents=valuestudents + value.size();
					groupsTable.getContainerProperty(itemid,"students").setValue(newvaluestudents);
					refreshStudents(idgroup);

				}
			}
		});
		
		removestudentButton.addListener(new Button.ClickListener() {
			@Override
			public void buttonClick(ClickEvent event) {

				System.out.println("remove");
				Set<?> value = (Set<?>) studentsGroupTable.getValue();
				if (null != value && value.size() > 0) {
					Iterator it=value.iterator();
					Integer idgroup =(Integer) groupsTable.getValue();
					while(it.hasNext()){
						Integer idstudent=(Integer) it.next();
						app.storage.deleteStudentFromGroup(idgroup,idstudent);
					}
					Integer itemid=(Integer) groupsTable.getValue();
					Integer valuestudents =(Integer) groupsTable.getContainerProperty(itemid,"students").getValue();
					Integer newvaluestudents=valuestudents - value.size();
					groupsTable.getContainerProperty(itemid,"students").setValue(newvaluestudents);
					refreshStudents(idgroup);
				}
			}
		});

	}
	public void refreshLayout(){
		refreshGroups() ;
		refreshTeachers();
	}
	private void refreshGroups() {
		Integer id= (Integer) groupsTable.getValue();
		app.storage.refreshTable(app.storage.getGroupsQueryContainer(), groupsTable);
		if (id!=null){groupsTable.setValue(id);}
	}
	private void refreshStudents(Integer idgroup) {
		app.storage.refreshTable(app.storage.getStudentsOfGroupQueryContainer(idgroup), studentsGroupTable);
		app.storage.refreshTable(app.storage.getStudentsNotInGroupQueryContainer(idgroup), studentsNoGroupTable);
	}
	public void refreshTeachers(){
		Collection<Teacher> collection = app.storage.allTeachers();
		IndexedContainer ic=new IndexedContainer();
		ic.addContainerProperty("fullname", String.class, null);
		Iterator<Teacher> it =collection.iterator();
		Item item=ic.addItem(0);
		item.getItemProperty("fullname").setValue(app.locale.getString("GROUPMANAGERLAYOUT_COMBOBOX_DEFAULTCAPTION"));
		while (it.hasNext()){
			Teacher teacher=it.next();
			item=ic.addItem(teacher.getID());
			item.getItemProperty("fullname").setValue(teacher.getFullname());
		}
		
		((AbstractSelect) newgroupWindow.groupForm.getField("teacher")).setContainerDataSource(ic);
		((AbstractSelect) editgroupWindow.groupForm.getField("teacher")).setContainerDataSource(ic);
	}

	
	/*
	 * Inner classes
	 */
	public class GroupsInfo {

		public String name = "";
		public String comment = "";
		public String teacher="";

		public String getName() {
			return name;
		}
		public void setName(String name) {
			this.name = name;
		}
		public String getComment() {
			return comment;
		}
		public void setComment(String comment) {
			this.comment = comment;
		}
		
		public String getTeacher() {
			return teacher;
		}
		public void setTeacher(String teacher) {
			this.teacher = teacher;
		}
	}
	private class GroupFieldFactory extends DefaultFieldFactory {

		final ComboBox teachers = new ComboBox(app.locale.getString("TEACHERSMANAGERLAYOUT_TEACHERSPANEL_CAPTION"));

		public GroupFieldFactory(AttendancemanagerApplication app){
			Collection<Teacher> collection = app.storage.allTeachers();
			IndexedContainer ic=new IndexedContainer();
			ic.addContainerProperty("fullname", String.class, null);
			Iterator<Teacher> it =collection.iterator();
			Item item=ic.addItem(0);
			item.getItemProperty("fullname").setValue(app.locale.getString("GROUPMANAGERLAYOUT_COMBOBOX_DEFAULTCAPTION"));
			while (it.hasNext()){
				Teacher teacher=it.next();
				item=ic.addItem(teacher.getID());
				item.getItemProperty("fullname").setValue(teacher.getFullname());
			}
			teachers.setContainerDataSource(ic);
			teachers.setImmediate(true);
			teachers.setItemCaptionPropertyId("fullname");
			teachers.setRequired(true);
			teachers.setInvalidAllowed(false);
			teachers.setNullSelectionAllowed(false);
			teachers.setFilteringMode(ComboBox.FILTERINGMODE_CONTAINS);
			teachers.setReadThrough(true);
			teachers.setValue(0);
			
		}
		
	    @Override
	    public Field createField(Item item, Object propertyId,
	            Component uiContext) {
            if ("teacher".equals(propertyId)) {
                // filtering ComboBox w/ country names
                return teachers;
            }

	        Field f = super.createField(item, propertyId, uiContext);
	        if ("name".equals(propertyId)) {
	            TextField tf = (TextField) f;
	            tf.setRequired(true);
	            tf.setRequiredError(app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_NAMEREQUIREDERROR"));
	            tf.addValidator(new StringLengthValidator(
	            		app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_NAMETOOLONGERROR")+
	            		" " + Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"), 1, Constants.SHORT_VARCHAR_LENGTH, false));
	        } else if ("comment".equals(propertyId)) {
	            TextField tf = (TextField) f;
	            tf.setRequired(true);
	            tf.setRequiredError(app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_COMMENTREQUIREDERROR"));
	            tf.addValidator(new StringLengthValidator(app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_COMMENTTOOLONGERROR")+
	            		" " + Constants.LONG_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"), 1, Constants.LONG_VARCHAR_LENGTH, false));
	        } 
	        return f;
	    }
	}	
	
	private class NewGroupWindow extends Window {
		
		private final Form groupForm = new Form();
		private final GroupsInfo groupInfo = new GroupsInfo();
		
		public NewGroupWindow() {

			
			BeanItem<GroupsInfo> groupBean = new BeanItem<GroupsInfo>(groupInfo);
			groupForm.setItemDataSource(groupBean,Arrays.asList ( "name","teacher","comment"));
			
			groupForm.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_FORMTITLE_CAPTION"));
			groupForm.setWriteThrough(false); // we want explicit 'apply'
			groupForm.setInvalidCommitted(false); // no invalid values in datamodel

	        // FieldFactory for customizing the fields and adding validators
			//System.out.println("Creando formulario");
			groupForm.setFormFieldFactory(new GroupFieldFactory(app));
			groupForm.setItemDataSource(groupBean); // bind to POJO via BeanItem

	        // Determines which properties are shown, and in which order:
			groupForm.setVisibleItemProperties(Arrays.asList(new String[] {
	                "name", "comment", "teacher"}));
			groupForm.setValidationVisible(true);
		
			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					if (!groupForm.isValid()) return;
					Group group = new Group(0, (String)groupForm.getField("name").getValue(), (String)groupForm.getField("comment").getValue(),(Integer)groupForm.getField("teacher").getValue());
					Integer idnewGroup=app.storage.addGroup(group);
					refreshGroups();
					groupsTable.setValue(idnewGroup);
                    getParent().getWindow().showNotification(
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_GROUPADDED")+" "+(String)groupForm.getField("name").getValue() ,
                            Notification.TYPE_TRAY_NOTIFICATION);

                    setValuesEmpty();
                    close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					groupForm.getField("name").setValue("");
					groupForm.getField("comment").setValue("");
					close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			groupForm.getFooter().addComponent(hl);
			hl.setSpacing(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			
    		addComponent(groupForm);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
    		
			groupForm.getField("teacher").setCaption(app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_MANAGERTEACHER"));
		}
		public void setValuesEmpty(){
			groupForm.getField("name").setValue("");
			groupForm.getField("comment").setValue("");
			groupForm.getField("teacher").setValue(0);
		}
		
	}
	
	private class EditGroupWindow extends Window {
		
		private final Form groupForm = new Form();
		private final GroupsInfo groupInfo = new GroupsInfo();
		private Integer idgroup;
		
		public EditGroupWindow() {
			
			
			BeanItem<GroupsInfo> groupBean = new BeanItem<GroupsInfo>(groupInfo);
			groupForm.setItemDataSource(groupBean,Arrays.asList ( "name","teacher","comment"));
			
			groupForm.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_FORMTITLE_CAPTION"));
			groupForm.setWriteThrough(false); // we want explicit 'apply'
			groupForm.setInvalidCommitted(false); // no invalid values in datamodel

	        // FieldFactory for customizing the fields and adding validators
			//System.out.println("Creando formulario");
			groupForm.setFormFieldFactory(new GroupFieldFactory(app));
			groupForm.setItemDataSource(groupBean); // bind to POJO via BeanItem

	        // Determines which properties are shown, and in which order:
			groupForm.setVisibleItemProperties(Arrays.asList(new String[] {
	                "name", "comment", "teacher"}));
			groupForm.setValidationVisible(true);
		
			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					if (!groupForm.isValid()) return;
					Integer idgroup =(Integer) groupsTable.getValue();
					Group group = new Group(idgroup, (String)groupForm.getField("name").getValue(), (String)groupForm.getField("comment").getValue(), (Integer)groupForm.getField("teacher").getValue());
					app.storage.updateGroup(group);
                    getParent().getWindow().showNotification(
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_GROUPEDITED")+" "+(String)groupForm.getField("name").getValue() ,
                            Notification.TYPE_TRAY_NOTIFICATION);
					groupForm.getField("name").setValue("");
					groupForm.getField("comment").setValue("");
					groupForm.getField("teacher").setValue(0);
					refreshGroups();
					groupsTable.setValue(idgroup);
					close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					groupForm.getField("name").setValue("");
					groupForm.getField("comment").setValue("");
					close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			groupForm.getFooter().addComponent(hl);
			hl.setSpacing(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			
    		addComponent(groupForm);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}
		public void setIdGroup(Integer idgroup){
			this.idgroup=idgroup;
		}
		
	}
	
	private class DeleteGroupWindow extends Window {
		
		private Group group=new Group(0,"","",0);
		private Label messageLabel;

		public DeleteGroupWindow() {
			Embedded warningEmbedded = new Embedded(null,new ThemeResource("img/Warning_icon.png"));
			 
			messageLabel = new Label();
			String messageString = app.locale.getString("GROUPMANAGERLAYOUT_DELETEWINDOW_MESSAGE");
			messageLabel.setValue(messageString);
			messageLabel.setContentMode(Label.CONTENT_XHTML);
			
			HorizontalLayout warningLayout = new HorizontalLayout();
			warningLayout.setSpacing(true);
			warningLayout.addComponent(warningEmbedded);
			warningLayout.addComponent(messageLabel);
			warningLayout.setSizeFull();
			
			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					app.storage.deleteGroup(group);
                    getParent().getWindow().showNotification(
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_GROUPDELETED")+" "+(String)group.getName(),
                    		Notification.TYPE_TRAY_NOTIFICATION);
					refreshGroups();
					close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.setMargin(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			hl.setSizeFull();
			addComponent(warningLayout);
    		addComponent(hl);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}
		
		public void setGroup(Group group){
			this.group=group;
		}
		
	}
	
	/**
	 * Muestra la ventana que permite añadir al grupo seleccionado los estudiantes
	 * de otro grupo. (Sin repetirlos)
	 * 
	 * @author vcarceler
	 *
	 */
	private class IncludeGroupWindow extends Window {
		
		private ComboBox groupsComboBox;
		
		@SuppressWarnings("unused")
		public IncludeGroupWindow() {
			
			Label messageLabel = new Label();
			String messageString = app.locale.getString("GROUPMANAGERLAYOUT_INCLUDEWINDOW_MESSAGE");
			messageLabel.setValue(messageString);
			messageLabel.setContentMode(Label.CONTENT_XHTML);
			
			groupsComboBox = new ComboBox(app.locale.getString("GROUPMANAGERLAYOUT_GROUPCOMBO_CAPTION"));
			
			Collection<Group> groups = storage.allGroups();
			IndexedContainer ic = new IndexedContainer();
			ic.addContainerProperty("name", String.class, null);
			for(Group group : groups) {
				Item item = ic.addItem(group.getID());
				item.getItemProperty("name").setValue(group.getName());
			}
			
			groupsComboBox.setContainerDataSource(ic);
			groupsComboBox.setImmediate(true);
			groupsComboBox.setItemCaptionPropertyId("name");
			groupsComboBox.setNullSelectionAllowed(false);
			groupsComboBox.setFilteringMode(ComboBox.FILTERINGMODE_CONTAINS);
		 	
			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					int idgroupA = (Integer)groupsTable.getValue();
					int idgroupB = (Integer)groupsComboBox.getValue();
					
					int rows = storage.addGroupToGroup(idgroupA, idgroupB);
					
					refreshStudents(idgroupA);
					getParent().getWindow().showNotification(
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_GROUPINCLUDED") + " " + rows,
                    		Notification.TYPE_TRAY_NOTIFICATION);
					close();
				}
			});
			
			
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					close();
				}
			});
			
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.setMargin(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			//hl.setSizeUndefined();
			
			addComponent(messageLabel);
			addComponent(groupsComboBox);
			addComponent(hl);
			
			VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}
	}
	
	/**
	 * Ventana emergente de importación de datos en formato csv.
	 * 
	 * @author apalacios
	 *
	 */
	private class ImportGroupWindow extends Window implements Upload.SucceededListener, Upload.FailedListener{
		
		private CSVUploader csvuploader = new CSVUploader(Group.class.getConstructor(String.class, String.class), Group.getBasicProperties());
		private Upload upload = new Upload(app.locale.getString("GROUPMANAGERLAYOUT_UPLOAD_CAPTION"), csvuploader);

		/**
		 * Constructor de la ventana de importación de csv. Crea un objeto Upload y lo conecta con el CSVUploader.
		 * @throws SecurityException
		 * @throws NoSuchMethodException
		 */
		public ImportGroupWindow() throws SecurityException, NoSuchMethodException {
			
			String alert = app.locale.getString("GROUPMANAGERLAYOUT_UPLOADALERT_MESSAGE");
			
			csvuploader.setAlert(alert);
			
			addComponent(upload);			
			
			upload.addListener((Upload.FinishedListener)csvuploader);
			upload.addListener((Upload.SucceededListener)this);
			upload.addListener((Upload.FailedListener)this);
			
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				@Override
				public void buttonClick(ClickEvent event) {
					close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.setMargin(true);

			hl.addComponent(cancelButton);
    		addComponent(hl);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}

		/**
		 * Evento de carga correcta. Muestra una notificación con el número de registros correctos y fallidos.
		 */
		@Override
		public void uploadSucceeded(SucceededEvent event) {
			refreshGroups();
			close();
			System.out.println("Grupos cargados !!!");

			String finishMessage = app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESS_TAGMESSAGE")+" "
				+ csvuploader.successfulRecordsCounter + " "
				+ app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESSOK_MESSAGE")
				+ app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESS_TAGMESSAGE")+" "
				+ csvuploader.failedRecordsCounter + " "
				+ app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESSNOK_MESSAGE");
			
			Label finishLabel = csvuploader.getLabelFromString(finishMessage);											
			AttendancemanagerApplication.adminWindow.showNotification(app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESS_MESSAGE"),(String) finishLabel.getValue(), Notification.TYPE_TRAY_NOTIFICATION);

		}

		/**
		 * Evento de carga fallida. Si la carga falla, muestra una alerta de error.
		 */
		@Override
		public void uploadFailed(FailedEvent event) {
			System.out.println("Ohhhh, no he podido insertar los grupos !!!");

			String errorMessage = app.locale.getString("GROUPMANAGERLAYOUT_UPLOADFAILED_ERRORSTRING");
			Label errorLabel = csvuploader.getLabelFromString(errorMessage);			
			AttendancemanagerApplication.adminWindow.showNotification(app.locale.getString("GROUPMANAGERLAYOUT_UPLOADFAILED_MESSAGE"),(String) errorLabel.getValue(), Notification.TYPE_ERROR_MESSAGE);		

		}		
	}
}
