/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

/*
 * A layout with different tabs to manage groups, students, teachers, ....
 */

package org.iespuigcastellar.attendancemanager.screenlayouts;

import java.util.ResourceBundle;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;
import org.iespuigcastellar.attendancemanager.core.Logger;

import com.vaadin.terminal.ThemeResource;
import com.vaadin.ui.TabSheet;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.TabSheet.SelectedTabChangeEvent;
import com.vaadin.ui.TabSheet.Tab;

/**
 * 
 * @author vcarceler
 *
 */
@SuppressWarnings("serial")
public class AdminLayout extends VerticalLayout implements TabSheet.SelectedTabChangeListener {
	
	private TabSheet tabsTabSheet = new TabSheet();
	
	private TeacherMainLayout teachermainlayout = new TeacherMainLayout(true);
	private ClassblockManagerLayout classblockmanagerlayout = new ClassblockManagerLayout();
	private GroupManagerLayout groupmanagerlayout = new GroupManagerLayout();
	private TeachersManagerLayout teachermanagerlayout =new TeachersManagerLayout();
	private StudentsManagerLayout studentmanagerlayout = new StudentsManagerLayout();
	private JustifyMissesLayout justifymisseslayout = null;
	private TermManagerLayout termmanagerlayout = new TermManagerLayout();
	private SubjectManagerLayout subjectmanagerlayout = new SubjectManagerLayout();
	// Substituyo ReportsLayout por un ReportLayout
	//private ReportsLayout reportslayout = new ReportsLayout();
	private ReportLayout reportlayout = new ReportLayout();
	private MessageLayout messageslayout = new MessageLayout(true);
	private NotifierEmailSmsLayout notifierlayout=new NotifierEmailSmsLayout(true);
	
	//private AttendancemanagerApplication app;
	private ResourceBundle locale = AttendancemanagerApplication.locale;
	
	private void initLayout() {
		
		tabsTabSheet.addListener(this);
		
		TabSheet.Tab tab;

		tab = tabsTabSheet.addTab(teachermainlayout, locale.getString("ADMINLAYOUT_TEACHERMAINTAB_CAPTION"), null);
		tab.setDescription(locale.getString("ADMINLAYOUT_TEACHERMAINTAB_DESCRIPTION"));
 		tab.setIcon(new ThemeResource("img/house.png"));


		tab = tabsTabSheet.addTab(studentmanagerlayout, locale.getString("ADMINLAYOUT_STUDENTSMANAGERTAB_CAPTION"), null);
		tab.setDescription(locale.getString("ADMINLAYOUT_STUDENTSMANAGERTAB_DESCRIPTION"));
 		tab.setIcon(new ThemeResource("img/user.png"));
		
		tab = tabsTabSheet.addTab(groupmanagerlayout, locale.getString("ADMINLAYOUT_GROUPMANAGERTAB_CAPTION"), null);
		tab.setDescription(locale.getString("ADMINLAYOUT_GROUPMANAGERTAB_DESCRIPTION"));
 		tab.setIcon(new ThemeResource("img/group.png"));

		tab = tabsTabSheet.addTab(teachermanagerlayout, locale.getString("ADMINLAYOUT_TEACHERSMANAGERTAB_CAPTION"), null);
		tab.setDescription(locale.getString("ADMINLAYOUT_TEACHERSMANAGERTAB_DESCRIPTION"));
 		tab.setIcon(new ThemeResource("img/user_gray.png"));

		tab = tabsTabSheet.addTab(termmanagerlayout, locale.getString("ADMINLAYOUT_TERMSMANAGERTAB_CAPTION"), null);
		tab.setDescription(locale.getString("ADMINLAYOUT_TERMSMANAGERTAB_DESCRIPTION"));
 		tab.setIcon(new ThemeResource("img/date.png"));
		//if(tab == null) Logger.log("AdminLayout: Error al añadir la pestaña TermManagerLayout");

		tab = tabsTabSheet.addTab(subjectmanagerlayout, locale.getString("ADMINLAYOUT_SUBJECTSMANAGERTAB_CAPTION"), null);
		tab.setDescription(locale.getString("ADMINLAYOUT_SUBJECTSMANAGERTAB_DESCRIPTION"));
 		tab.setIcon(new ThemeResource("img/book.png"));
	
		tab = tabsTabSheet.addTab(classblockmanagerlayout, locale.getString("ADMINLAYOUT_CLASSBLOCKMANAGERTAB_CAPTION"), null);
		tab.setDescription(locale.getString("ADMINLAYOUT_CLASSBLOCKMANAGERTAB_DESCRIPTION"));
 		tab.setIcon(new ThemeResource("img/clock.png"));
		
 		Logger.log("AdminLayout: Build JustifyMissesLayout");
 		justifymisseslayout = new JustifyMissesLayout(true);
		tab = tabsTabSheet.addTab(justifymisseslayout, locale.getString("ADMINLAYOUT_JUSTIFYMANAGERTAB_CAPTION"), null);
		tab.setDescription(locale.getString("ADMINLAYOUT_JUSTIFYMANAGERTAB_DESCRIPTION"));
 		tab.setIcon(new ThemeResource("img/flag_yellow.png"));

 		// ReportsLayout ha sido substituido por ReportLayout
//		tab = tabsTabSheet.addTab(reportslayout, locale.getString("ADMINLAYOUT_REPORTSTAB_CAPTION"), null);
//		tab.setDescription(locale.getString("ADMINLAYOUT_REPORTSTAB_DESCRIPTION"));
// 		tab.setIcon(new ThemeResource("img/report.png"));
 		
 		tab = tabsTabSheet.addTab(reportlayout, locale.getString("ADMINLAYOUT_REPORTSTAB_CAPTION"), null);
		tab.setDescription(locale.getString("ADMINLAYOUT_REPORTSTAB_DESCRIPTION"));
 		tab.setIcon(new ThemeResource("img/report.png"));
 		
 		CommunicationLayout communicationLayout = new CommunicationLayout(true);
	 	tab = tabsTabSheet.addTab(communicationLayout, locale.getString("ADMINLAYOUT_MESSAGESTAB_CAPTION"), null);
	 	tab.setDescription(locale.getString("ADMINLAYOUT_MESSAGESMANAGERTAB_DESCRIPTION"));
	 	tab.setIcon(new ThemeResource("img/email.png"));
	 		
//		tab = tabsTabSheet.addTab(messageslayout, locale.getString("ADMINLAYOUT_MESSAGESTAB_CAPTION"), null);
//		tab.setDescription(locale.getString("ADMINLAYOUT_MESSAGESMANAGERTAB_DESCRIPTION"));
// 		tab.setIcon(new ThemeResource("img/email.png"));
 		
 		tab = tabsTabSheet.addTab(notifierlayout, locale.getString("ADMINLAYOUT_NOTIFIERSMSEMAILTAB_CAPTION"), null);
		tab.setDescription(locale.getString("ADMINLAYOUT_NOTIFIERSMSEMAILTAB_DESCRIPTION"));
 		tab.setIcon(new ThemeResource("img/bell.png"));

		setSizeFull();
		tabsTabSheet.setSizeFull();		
	}
	
	public void attach() {
		//app = (AttendancemanagerApplication)getApplication();
		initLayout();
		addComponent(tabsTabSheet);
		this.setExpandRatio(tabsTabSheet, 1.0f);
		setSpacing(true);
	}

	@Override
	public void selectedTabChange(SelectedTabChangeEvent event) {

        TabSheet tabsheet = event.getTabSheet();
        Tab tab = tabsheet.getTab(tabsheet.getSelectedTab());
        if (tab != null) {
        	//getWindow().showNotification(locale.getString("ADMINLAYOUT_SELECTEDTAB_CAPTION")+ " " + tab.getCaption());
            if (tab.getCaption().equals(locale.getString("ADMINLAYOUT_CLASSBLOCKMANAGERTAB_CAPTION"))){

                  classblockmanagerlayout.refreshLayout();
            }else if (tab.getCaption().equals(locale.getString("ADMINLAYOUT_GROUPMANAGERTAB_CAPTION"))){
            	 groupmanagerlayout.refreshLayout();
            }else if (tab.getCaption().equals(locale.getString("ADMINLAYOUT_STUDENTSMANAGERTAB_CAPTION"))){
            	 studentmanagerlayout.refreshLayout();
            }else if (tab.getCaption().equals(locale.getString("ADMINLAYOUT_JUSTIFYMANAGERTAB_CAPTION"))){
            	justifymisseslayout.setLoading(true);
            	justifymisseslayout.refreshLayout();
            }
        }
        
	}

}
