/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.screenlayouts;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.Serializable;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.ResourceBundle;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;
import org.iespuigcastellar.attendancemanager.core.CreatePDF;
import org.iespuigcastellar.attendancemanager.core.DatabaseStorage;
import org.iespuigcastellar.attendancemanager.core.DownloadResource;
import org.iespuigcastellar.attendancemanager.core.Group;
import org.iespuigcastellar.attendancemanager.core.Logger;
import org.iespuigcastellar.attendancemanager.core.Student;
import org.iespuigcastellar.attendancemanager.core.Term;

import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.util.BeanItem;
import com.vaadin.data.util.IndexedContainer;
import com.vaadin.ui.Button;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.Component;
import com.vaadin.ui.DefaultFieldFactory;
import com.vaadin.ui.Field;
import com.vaadin.ui.Form;
import com.vaadin.ui.InlineDateField;
import com.vaadin.ui.Panel;
import com.vaadin.ui.PopupDateField;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Window.Notification;

/**
 * 
 * @author vcarceler@iespuigcastellar.xeill.net
 * 
 */
@SuppressWarnings("serial")
public class ReportLayout extends VerticalLayout {
	private ResourceBundle locale = AttendancemanagerApplication.locale;
	private DatabaseStorage storage = AttendancemanagerApplication.storage;

	private ReportDetails details = new ReportDetails(); // POJO

	private final int REPORT_ID_TYPE1 = 1;
	private final int REPORT_ID_TYPE2 = 2;
	private final int REPORT_ID_TYPE3 = 3;
	private final int REPORT_ID_TYPE4 = 4;
	private final int REPORT_ID_TYPE5 = 5;
	private final int REPORT_ID_TYPE6 = 6;
	private final String REPORT_NAME_TYPE1 = locale.getString("REPORTSLAYOUT_LOADREPORTS_REPORT1");
	private final String REPORT_NAME_TYPE2 = locale.getString("REPORTSLAYOUT_LOADREPORTS_REPORT2");
	private final String REPORT_NAME_TYPE3 = locale.getString("REPORTSLAYOUT_LOADREPORTS_REPORT3");
	private final String REPORT_NAME_TYPE4 = locale.getString("REPORTSLAYOUT_LOADREPORTS_REPORT4");
	private final String REPORT_NAME_TYPE5 = locale.getString("REPORTSLAYOUT_LOADREPORTS_REPORT5");
	private final String REPORT_NAME_TYPE6 = locale.getString("REPORTSLAYOUT_LOADREPORTS_REPORT6");

	public ReportLayout() {

		// POJO and associated Item
		BeanItem<ReportDetails> detailsItem = new BeanItem<ReportDetails>(details);

		// FORM
		final Form detailsForm = new FormHelper(detailsItem);
		Panel formPanel = new Panel(locale.getString("REPORTSLAYOUT_PANELCONTENTS_CAPTION"));
		formPanel.setStyleName("bubble");
		formPanel.addComponent(detailsForm);
		formPanel.setSizeFull();
		addComponent(formPanel);

		// Execute button
		Button ok = new Button("Ok", new Button.ClickListener() {

			@Override
			public void buttonClick(ClickEvent event) {
				detailsForm.commit();
				if (checkDetails())
					generateReport();
			}
		});

		detailsForm.getFooter().setMargin(true);
		detailsForm.getFooter().addComponent(ok);

		setSizeFull();
	}

	/**
	 * Checks details and shows warnings to the user, if everything ok returns
	 * true.
	 * 
	 * @return
	 */
	private boolean checkDetails() {
		boolean result = true;

		if ((details.getReport()!=REPORT_ID_TYPE6) && (details.getGroup() == null)) {
			getWindow()
					.showNotification(
							locale.getString("REPORTSLAYOUT_INVALIDGROUPERROR_CAPTION"),
							locale.getString("REPORTSLAYOUT_INVALIDGROUPERROR_DESCRIPTION"),
							Notification.TYPE_ERROR_MESSAGE);
			return false;
		}
		
		if ((details.getReport()==REPORT_ID_TYPE6) && (details.getStudent() == null)) {
			getWindow()
					.showNotification(
							locale.getString("REPORTSLAYOUT_INVALIDSTUDENTERROR_CAPTION"),
							locale.getString("REPORTSLAYOUT_INVALIDSTUDENTERROR_DESCRIPTION"),
							Notification.TYPE_ERROR_MESSAGE);
			return false;
		}

		if (details.getStart() == null) {
			getWindow()
					.showNotification(
							locale.getString("REPORTSLAYOUT_INVALIDSTARTDATE_CAPTION"),
							locale.getString("REPORTSLAYOUT_INVALIDSTARTDATE_DESCRIPTION"),
							Notification.TYPE_ERROR_MESSAGE);
			return false;
		}

		if (details.getEnd() == null) {
			getWindow()
					.showNotification(
							locale.getString("REPORTSLAYOUT_INVALIDENDDATE_CAPTION"),
							locale.getString("REPORTSLAYOUT_INVALIDENDDATE_DESCRIPTION"),
							Notification.TYPE_ERROR_MESSAGE);
			return false;
		}

		return result;
	}

	private void generateReport() {

		CreatePDF pdf = new CreatePDF();
		ByteArrayOutputStream baos = null;
		ByteArrayInputStream bais;
		DownloadResource dr;

		try {
			switch (details.getReport()) {
			case REPORT_ID_TYPE1:
				baos = pdf.createReport1(details.getGroup().getID(),
						details.getStart(), details.getEnd());
				break;
			case REPORT_ID_TYPE2:
				baos = pdf.createReport2(details.getGroup().getID(),
						details.getStart(), details.getEnd());
				break;
			case REPORT_ID_TYPE3:
				baos = pdf.createReport3(details.getGroup().getID(),
						details.getStart(), details.getEnd());
				break;
			case REPORT_ID_TYPE4:
				baos = pdf.createReport4(details.getGroup().getID(),
						details.getStart(), details.getEnd());
				break;
			case REPORT_ID_TYPE5:
				baos = pdf.createReport5(details.getGroup().getID(),
						details.getStart(), details.getEnd());
				break;
			case REPORT_ID_TYPE6:
				baos = pdf.createReport6(details.getStudent(),
						details.getStart(), details.getEnd());
				break;
			}	
			
			

			bais = new ByteArrayInputStream(baos.toByteArray());
			dr = new DownloadResource(bais, getApplication(), "report-"
					+ System.currentTimeMillis() + ".pdf",
					AttendancemanagerApplication.MIME_PDF);
			getWindow().open(dr);

		} catch (Exception e) {
			Logger.log("ReportLayout.generateReport()\n" + details + "\n");
			e.printStackTrace();
		}
	}

	// private void initLayout() {
	//
	// }
	//
	// public void attach() {
	//
	// }

	// POJO
	public class ReportDetails implements Serializable {

		// private static final long serialVersionUID = 8897174727693486105L;

		private int report;
		private Group group;
		private Student student;
		private Term term;
		private Date start;
		private Date end;

		public int getReport() {
			return report;
		}

		public void setReport(int report) {
			this.report = report;
		}

		public Group getGroup() {
			return group;
		}

		public void setGroup(Group group) {
			this.group = group;
		}

		public Student getStudent() {
			return student;
		}

		public void setStudent(Student student) {
			this.student = student;
		}

		public Term getTerm() {
			return term;
		}

		public void setTerm(Term term) {
			this.term = term;
		}

		public Date getStart() {
			return start;
		}

		public void setStart(Date start) {
			this.start = start;
		}

		public Date getEnd() {
			return end;
		}

		public void setEnd(Date end) {
			this.end = end;
		}

		@Override
		public String toString() {
			return "ReportDetails [report=" + report + ", group=" + group
					+ ", student=" + student + ", term=" + term + ", start="
					+ start + ", end=" + end + "]";
		}
	}

	public class ReportDetailsFieldFactory extends DefaultFieldFactory {

		final ComboBox reportComboBox = new ComboBox(
				locale.getString("REPORTSLAYOUT_REPORTCOMBOBOX_CAPTION"));
		final ComboBox groupComboBox = new ComboBox(
				locale.getString("REPORTSLAYOUT_GROUPCOMBOBOX_CAPTION"));
		final ComboBox studentComboBox = new ComboBox(
				locale.getString("REPORTSLAYOUT_STUDENTCOMBOBOX_CAPTION"));
		final ComboBox termComboBox = new ComboBox(
				locale.getString("REPORTSLAYOUT_TERMCOMBOBOX_CAPTION"));
		final PopupDateField startDateField = new PopupDateField(
				locale.getString("REPORTSLAYOUT_STARTDATE_CAPTION"));
		final PopupDateField endDateField = new PopupDateField(
				locale.getString("REPORTSLAYOUT_ENDDATE_CAPTION"));

		// final Item nullTermItem;
		final Term nullTerm;

		public ReportDetailsFieldFactory() {

			// NullValidator notNullValidator = new
			// NullValidator("No puede ser nulo", false);

			// reportComboBox.setWidth("100%");
			reportComboBox.setFilteringMode(ComboBox.FILTERINGMODE_CONTAINS);

			IndexedContainer ic = new IndexedContainer();
			ic.addContainerProperty("reportname", String.class, null);

			Item item;

			item = ic.addItem(REPORT_ID_TYPE1);
			item.getItemProperty("reportname").setValue(REPORT_NAME_TYPE1);
			item = ic.addItem(REPORT_ID_TYPE2);
			item.getItemProperty("reportname").setValue(REPORT_NAME_TYPE2);
			item = ic.addItem(REPORT_ID_TYPE3);
			item.getItemProperty("reportname").setValue(REPORT_NAME_TYPE3);
			item = ic.addItem(REPORT_ID_TYPE5);
			item.getItemProperty("reportname").setValue(REPORT_NAME_TYPE5);
			item = ic.addItem(REPORT_ID_TYPE4);
			item.getItemProperty("reportname").setValue(REPORT_NAME_TYPE4);
			item = ic.addItem(REPORT_ID_TYPE6);
			item.getItemProperty("reportname").setValue(REPORT_NAME_TYPE6);

			reportComboBox.setContainerDataSource(ic);
			reportComboBox.setItemCaptionPropertyId("reportname");
			reportComboBox.setValue(REPORT_ID_TYPE1);
			reportComboBox.setNullSelectionAllowed(false);

			reportComboBox.addListener(new Property.ValueChangeListener() {

				@Override
				public void valueChange(ValueChangeEvent event) {
					 if(reportComboBox.getValue().equals(Integer.valueOf(REPORT_ID_TYPE6)))
					 {
						 groupComboBox.setEnabled(false);
						 studentComboBox.setEnabled(true);
					 } else {
						 groupComboBox.setEnabled(true);
						 studentComboBox.setEnabled(false);
					 }
				}
			});

			// groupComboBox.setWidth("100%");
			groupComboBox.setFilteringMode(ComboBox.FILTERINGMODE_CONTAINS);

			Collection<Group> groups;
			// Carga selectiva de grupos, o todos si se es administrador o
			// grupos de los que se es tutor.
			// if(AttendancemanagerApplication.user.getAdmin()) {
			// groups = storage.allGroups();
			// } else {
			// groups = storage.allGroups(AttendancemanagerApplication.user);
			// }
			// De momento cargamos todos los grupos para que cualquier
			// profesor pueda obtener informes de cualquier grupo.
			groups = storage.allGroups();

			ic = new IndexedContainer();
			ic.addContainerProperty("groupname", String.class, null);

			for (Group g : groups) {
				item = ic.addItem(g);
				item.getItemProperty("groupname").setValue(g.getName());
			}

			groupComboBox.setContainerDataSource(ic);
			groupComboBox.setItemCaptionPropertyId("groupname");
			groupComboBox.setNullSelectionAllowed(false);
			// groupComboBox.addValidator(notNullValidator);

			studentComboBox.setFilteringMode(ComboBox.FILTERINGMODE_CONTAINS);
			Collection<Student> students = storage.getStudentsCollection(0);
			ic = new IndexedContainer();
			ic.addContainerProperty("studentname", String.class, null);
			for (Student s : students) {
				item = ic.addItem(s);
				item.getItemProperty("studentname").setValue(s.getFullName());
			}
			studentComboBox.setContainerDataSource(ic);
			studentComboBox.setItemCaptionPropertyId("studentname");
			studentComboBox.setNullSelectionAllowed(false);
			studentComboBox.setEnabled(false);

			// termComboBox.setWidth("100%");
			termComboBox.setFilteringMode(ComboBox.FILTERINGMODE_CONTAINS);
			termComboBox.setNullSelectionAllowed(true);

			Collection<Term> terms = storage.allTerms();

			ic = new IndexedContainer();
			ic.addContainerProperty("termname", String.class, null);
			ic.addContainerProperty("term", Term.class, null);
			nullTerm = new Term(0,
					locale.getString("REPORTSLAYOUT_COMBOBOX_NOSELECTION"),
					null, null, null);
			item = ic.addItem(nullTerm);
			item.getItemProperty("termname").setValue(
					locale.getString("REPORTSLAYOUT_COMBOBOX_NOSELECTION"));
			termComboBox.setNullSelectionItemId(item);

			for (Term t : terms) {
				item = ic.addItem(t);
				item.getItemProperty("termname").setValue(t.getName());
				item.getItemProperty("term").setValue(t);
			}

			termComboBox.setContainerDataSource(ic);
			termComboBox.setItemCaptionPropertyId("termname");
			termComboBox.addListener(new Property.ValueChangeListener() {

				@Override
				public void valueChange(ValueChangeEvent event) {
					Term term = (Term) termComboBox.getValue();
					if (term != null) {
						if (term.getStart() != null)
							startDateField.setValue(term.getStart());
						if (term.getEnd() != null)
							endDateField.setValue(term.getEnd());
					}
				}
			});

			startDateField.setResolution(InlineDateField.RESOLUTION_DAY);
			// startDateField.addValidator(notNullValidator);
			startDateField.addListener(new Property.ValueChangeListener() {

				/**
				 * Si se cambia la fecha de inicio a un valor distinto del
				 * marcado por el term, se deselecciona el term
				 */
				@Override
				public void valueChange(ValueChangeEvent event) {
					Term term = (Term) termComboBox.getValue();
					Date date = (Date) startDateField.getValue();

					if (term != null && term.getStart() != null && date != null) {
						if (!term.getStart().equals(date))
							termComboBox.select(nullTerm);
					}
				}
			});

			endDateField.setResolution(InlineDateField.RESOLUTION_DAY);
			// endDateField.addValidator(notNullValidator);
			endDateField.addListener(new Property.ValueChangeListener() {

				/**
				 * Si se cambia la fecha final a un valor distinto del marcado
				 * por el term, se deselecciona el term
				 */
				@Override
				public void valueChange(ValueChangeEvent event) {
					Term term = (Term) termComboBox.getValue();
					Date date = (Date) endDateField.getValue();

					if (term != null && term.getEnd() != null && date != null) {
						if (!term.getEnd().equals(date))
							termComboBox.select(nullTerm);
					}
				}
			});
		}

		public Field createField(Item item, Object propertyId,
				Component uiContext) {
			System.out.println("createField: propertyId: " + propertyId);
			if ("report".equals(propertyId)) {
				return reportComboBox;
			} else if ("group".equals(propertyId)) {
				return groupComboBox;
			} else if ("student".equals(propertyId)) {
				return studentComboBox;
			} else if ("term".equals(propertyId)) {
				return termComboBox;
			} else if ("start".equals(propertyId)) {
				return startDateField;
			} else if ("end".equals(propertyId)) {
				return endDateField;
			}

			Field f = super.createField(item, propertyId, uiContext);

			return f;
		}
	}

	// Form helper
	public class FormHelper extends Form {

		private VerticalLayout layout;

		public FormHelper(BeanItem<ReportDetails> item) {
			// setCaption(locale.getString("REPORTSLAYOUT_PANELCONTENTS_CAPTION"));

			// layout = new GridLayout(1, 2);
			layout = new VerticalLayout();
			layout.setMargin(true, false, false, true);
			layout.setSpacing(true);

			setLayout(layout);

			setWriteThrough(true);
			setImmediate(true);
			setInvalidCommitted(false);

			setFormFieldFactory(new ReportDetailsFieldFactory());
			setItemDataSource(item);

			setVisibleItemProperties(Arrays.asList(new String[] { "report",
					"group", "student", "term", "start", "end" }));
		}
	}
}
