package org.vaadin.hesara.customfield;

import com.vaadin.Application;
import com.vaadin.data.Item;
import com.vaadin.data.util.BeanItem;
import com.vaadin.ui.Button;
import com.vaadin.ui.Component;
import com.vaadin.ui.CustomField;
import com.vaadin.ui.DefaultFieldFactory;
import com.vaadin.ui.Field;
import com.vaadin.ui.Form;
import com.vaadin.ui.Label;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;

/**
 * A simple demonstration application for {@link CustomField}.
 * 
 * @author Henri Sara
 */
public class CustomFieldApplication extends Application {

    /**
     * An example of a custom field for editing a boolean value. The field is
     * composed of multiple components, and could also edit a more complex data
     * structures. Here, the commit etc. logic is not overridden.
     */
    public static class BooleanField extends CustomField {

        public BooleanField() {
            VerticalLayout layout = new VerticalLayout();
            
            layout.addComponent(new Label("Please click the button"));
            
            final Button button = new Button("Click me");
            button.addListener(new ClickListener() {
                public void buttonClick(ClickEvent event) {
                    Object value = getValue();
                    boolean newValue = true;
                    if ((value instanceof Boolean) && ((Boolean) value)) {
                        newValue = false;
                    }
                    setValue(newValue);
                    button.setCaption(newValue ? "On" : "Off");
                }
            });
            layout.addComponent(button);
            
            setCompositionRoot(layout);
        }

        @Override
        public Class<?> getType() {
            return Boolean.class;
        }
    }

    /**
     * Data model class with two boolean fields.
     */
    public static class TwoBooleans {
        private boolean normal;
        private boolean custom;

        public void setNormal(boolean normal) {
            this.normal = normal;
        }

        public boolean isNormal() {
            return normal;
        }

        public void setCustom(boolean custom) {
            this.custom = custom;
        }

        public boolean isCustom() {
            return custom;
        }
    }

    @Override
    public void init() {
        final Window mainWindow = new Window("Custom Field Example");

        final Form form = new Form();
        form.setCaption("Form with a customized boolean field");
        form.setFormFieldFactory(new DefaultFieldFactory() {
            @Override
            public Field createField(Item item, Object propertyId,
                    Component uiContext) {
                if ("custom".equals(propertyId)) {
                    return new BooleanField();
                }
                return super.createField(item, propertyId, uiContext);
            }
        });
        final TwoBooleans data = new TwoBooleans();
        form.setItemDataSource(new BeanItem<TwoBooleans>(data));

        mainWindow.addComponent(form);

        Button submit = new Button("Submit", new ClickListener() {
            public void buttonClick(ClickEvent event) {
                form.commit();
                mainWindow.showNotification("The custom value was "
                        + data.isCustom() + " and the normal value "
                        + data.isNormal());
            }
        });
        mainWindow.addComponent(submit);

        setMainWindow(mainWindow);
    }

}
